/**
 * The Equipment class holds all the things the kobold is currently wearing,
 * as well as some convenience functions which aggregate equipment-functionality
 * into a more usable format.
 */
function Equipment() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Turns a slot-enum into the appropriate member of this class and returns it.
     * @param {SlotEnum} slot The slot to convert.
     * @returns {string} The name of the member.
     */
    function getMember(slot) {
        return SlotEnum.properties[slot].member;
    }

    /**
     * Returns the item as a properly-constructed and loaded object, or null if the item is null.
     * @param {object} item The JSON-built item. 
     * @returns {object} The item as a properly-constructed and loaded object, or null if the item is null.
     */
    function itemOrNull(item) {
        if (item) {
            var properlyConstructedItem = ItemEnum.properties[item.id].create();
            properlyConstructedItem.load(item);
            return properlyConstructedItem;
        }
        return null;
    }

    /**
     * Converts the passed object or id into an object.
     * If it is already an object, it is returned.
     * If it is an item id, its constructor is fetched and called.
     * @param {object | ItemEnum} objectOrId The object or the id.
     * @returns {object} The (potentially constructed) object.
     */
    function convertToItem(objectOrId) {
        if (typeof objectOrId === "object") {
            return objectOrId;
        }
        return constructItem(objectOrId);
    }

    /**
     * Construct an item via its itemId and returns the item.
     * @param {ItemEnum} itemId The id of the item.
     * @returns {object} The constructed item.
     */
    function constructItem(itemId) {
        return ItemEnum.properties[itemId].create();
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.bra = null;
    this.helmet = null;
    this.pants = null;
    this.shirt = null;
    this.shoes = null;
    this.underwear = null;
    this.weapon = null;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Strips the item in the passed slot, setting it to null.
     * @param {SlotEnum} slot The slot to strip.
     */
    this.strip = function (slot) {
        var member = getMember(slot);
        that[member] = null;
    };

    /**
     * Removes all equipment from the kobold. Should be called AFTER fetching
     * and storing details of the stripping for display purposes.
     */
    this.stripNaked = function () {
        that.bra = null;
        that.helmet = null;
        that.pants = null;
        that.shirt = null;
        that.shoes = null;
        that.underwear = null;
        that.weapon = null;
    };

    /**
     * Returns true if the kobold is completely naked (including shoes and helmet).
     * @returns {boolean} True if the kobold is completely naked. False otherwise.
     */
    this.completelyNaked = function () {
        return !that.bra && !that.helmet && !that.pants && !that.shirt && !that.shoes && !that.underwear;
    };

    /**
     * Returns true if all of the kobold's most important parts are naked.
     * Important parts are: bra, pants, shirt and underwear.
     * @returns {boolean} True if all of the kobold's most important parts are naked.
     */
    this.importantNaked = function () {
        return !that.bra && !that.pants && !that.shirt && !that.underwear;
    };

    /**
     * Wears the passed piece of equipment in the appropriate slot.
     * Removes any previously worn piece of gear in that slot.
     * @param {Gear | ItemEnum} gear The piece of gear to wear. May be the id of the item instead.
     */
    this.wear = function (gear) {
        if (!gear) {
            return;
        }
        gear = convertToItem(gear);
        var slot = gear.slot;
        var member = getMember(slot);
        that[member] = gear;
    };

    /**
     * Returns true if the kobold is wearing a piece of equipment in the passed slot.
     * @param {SlotEnum} slot The slot to check.
     * @returns {boolean} True if the kobold is wearing something in the slot, false otherwise.
     */
    this.isWearing = function (slot) {
        var member = getMember(slot);
        return that[member] !== null;
    };

    /**
     * Returns true if the kobold is visibly wearing something not intended for its gender.
     * @returns {boolean} True if the kobold is wearing something not intended for its gender. False otherwise.
     */
    this.isCrossdressing = function () {
        var braWrongGender = that.bra && that.bra.wrongGender();
        var helmetWrongGender = that.helmet && that.helmet.wrongGender();
        var pantsWrongGender = that.pants && that.pants.wrongGender();
        var shirtWrongGender = that.shirt && that.shirt.wrongGender();
        var shoesWrongGender = that.shoes && that.shoes.wrongGender();
        var underwearWrongGender = that.underwear && that.underwear.wrongGender();

        if (that.pants && that.pants.coversGroin()) {
            underwearWrongGender = false;
        }
        if (that.shirt && that.shirt.coversChest()) {
            braWrongGender = false;
        }

        return braWrongGender || helmetWrongGender || pantsWrongGender || shirtWrongGender || shoesWrongGender || underwearWrongGender;
    };

    /**
     * Returns true if the kobold's groin is exposed and false otherwise.
     * @returns {boolean} True if the kobold's groin is exposed, false otherwise.
     */
    this.groinExposed = function () {
        var pantsCovered = false;
        if (that.pants && that.pants.coversGroin()) {
            pantsCovered = true;
        }
        var underwearCovered = false;
        if (that.underwear && that.underwear.coversGroin()) {
            underwearCovered = true;
        }
        var shirtCovered = false;
        if (that.shirt && that.shirt.coversGroin()) {
            shirtCovered = true;
        }
        return !pantsCovered && !underwearCovered && !shirtCovered;
    };

    /**
     * Returns true if the kobold's rear is exposed and false otherwise.
     * @returns {boolean} True if the kobold's rear is exposed, false otherwise.
     */
    this.rearExposed = function () {
        var pantsCovered = false;
        if (that.pants && that.pants.coversRear()) {
            pantsCovered = true;
        }
        var underwearCovered = false;
        if (that.underwear && that.underwear.coversRear()) {
            underwearCovered = true;
        }
        var shirtCovered = false;
        if (that.shirt && that.shirt.coversRear()) {
            shirtCovered = true;
        }
        return !pantsCovered && !underwearCovered && !shirtCovered;
    };

    /**
     * Returns true if the kobold's chest is exposed and false otherwise.
     * @returns {boolean} True if the kobold's chest is exposed, false otherwise.
     */
    this.chestExposed = function () {
        var shirtCovered = that.shirt && that.shirt.coversChest();
        var braCovered = that.bra !== null;
        return !shirtCovered && !braCovered;
    };

    /**
     * Rebuilds a piece of gear stored in the history object.
     * @param {object} object The history-saved object.
     * @returns {Gear} The gear constructed from the history-saved object. Null if the object is null.
     */
    this.buildGearFromHistory = function (object) {
        return itemOrNull(object);
    };

    /**
     * Loads this object from a type-stripped version thereof.
     * @param {Equipment-stripped} equipment An Equipment object, stripped of private members and all functions.
     */
    this.load = function (equipment) {
        that.bra = itemOrNull(equipment.bra);
        that.helmet = itemOrNull(equipment.helmet);
        that.pants = itemOrNull(equipment.pants);
        that.shirt = itemOrNull(equipment.shirt);
        that.shoes = itemOrNull(equipment.shoes);
        that.underwear = itemOrNull(equipment.underwear);
        that.weapon = itemOrNull(equipment.weapon);
    };
}
