/**
 * This class acts as a dedicated controller for the bandit part of the Forest scene.
 * @param {string} sceneId The id of the scene.
 */
function ForestBanditController(sceneId) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var id = sceneId;
    var sourceId = "banditSource";

    var arrivalId = "banditArrivalChoice";
    var arrivalWeapon = "banditArrivalWeapon";
    var arrivalStatus = "banditArrivalStatus";

    var rockId = "banditRockChoice";

    var rewardSource = "banditRewardSource";
    var rewardGoldId = "banditRewardGold";
    var rewardId = "banditRewardChoice";
    var rewardNameId = "banditRewardName";
    var rewardChangeId = "banditRewardChange";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";
    this.watch = new ForestBanditWatchController(sceneId);
    this.intervene = new ForestBanditInterveneController(sceneId);

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Sets the source scene.
     * @param {ForestBanditSourceEnum} source The source scene.
     */
    this.setSource = function (source) {
        var history = global.kobold.history.get(id);
        history[sourceId] = source;
        global.kobold.history.set(id, history);
    };

    /**
     * Fetches the source scene from history and returns it.
     * May return undefined if the source scene has not yet been set.
     * @returns {ForestBanditSourceEnum} The source scene.
     */
    this.getSource = function () {
        var history = global.kobold.history.get(id);
        return history[sourceId];
    };

    /**
     * Returns true if the source scene has been set.
     */
    this.hasSource = function () {
        return global.kobold.history.has(id, sourceId);
    };


    /**
     * Makes the arrival choice.
     * @param {ForestBanditArrivalChoiceEnum} choice The choice.
     */
    this.makeArrivalChoice = function (choice) {
        var history = global.kobold.history.get(id);
        history[arrivalId] = choice;
        global.kobold.history.set(id, history);
        if (choice == ForestBanditArrivalChoiceEnum.ROCK) {
            global.kobold.inventory.equipment.weapon = null;
        }
    };

    /**
     * Fetches the arrival choice from history and returns it.
     * May return undefined if the arrival choice has not yet been set.
     */
    this.getArrivalChoice = function () {
        var history = global.kobold.history.get(id);
        return history[arrivalId];
    };

    /**
     * Returns true if the arrival choice has been made.
     */
    this.hasMadeArrivalChoice = function () {
        return global.kobold.history.has(id, arrivalId);
    };

    /**
     * Increases the kobold's sanity and stamina if required.
     * @returns {string} HTML describing the change.
     */
    this.arrivalStatusChange = function () {
        if (!global.kobold.history.has(id, arrivalStatus)) {
            global.kobold.status.mental.increase(5, "Thought things over");
            global.kobold.status.stamina.increase(5, "Relaxing walk");
            var history = global.kobold.history.get(id);
            history[arrivalStatus] = true;
            global.kobold.history.set(id, history);
        }
        return global.ui.change.buildNotification([
            new Change("Time to think", "Sanity", +5),
            new Change("Relaxing walk", "Stamina", +5)
        ]);
    };

    /**
     * Stores (if required) and retrieves the kobold's weapon at the time of traversing the path,
     * compares it to ItemEnum.ROCK and returns the result.
     * @returns {Gear} The weapon.
     */
    this.arrivalHasRock = function () {
        if (!global.kobold.history.has(id, arrivalWeapon)) {
            var history = global.kobold.history.get(id);
            if (global.kobold.inventory.equipment.isWearing(SlotEnum.WEAPON)) {
                history[arrivalWeapon] = global.kobold.inventory.equipment.weapon.id;
            } else {
                history[arrivalWeapon] = 0;
            }
            global.kobold.history.set(id, history);
        }
        return global.kobold.history.get(id)[arrivalWeapon] === ItemEnum.ROCK;
    };


    /**
     * Makes the rock choice.
     * @param {ForestBanditRockChoiceEnum} choice The choice.
     */
    this.makeRockChoice = function (choice) {
        global.kobold.history.shared.tenPawGang.usedRock = true;
        if (choice === ForestBanditRockChoiceEnum.SCARYNOISE) {
            global.kobold.history.shared.tenPawGang.relation = TenPawGangEnum.SPOOKED;
        }
        var history = global.kobold.history.get(id);
        history[rockId] = choice;
        global.kobold.history.set(id, history);
    };

    /**
     * Fetches the arrival choice from history and returns it.
     * May return undefined if the arrival choice has not yet been set.
     */
    this.getRockChoice = function () {
        var history = global.kobold.history.get(id);
        return history[rockId];
    };

    /**
     * Returns true if the arrival choice has been made.
     */
    this.hasMadeRockChoice = function () {
        return global.kobold.history.has(id, rockId);
    };


    /**
     * Sets the source for the reward.
     * @param {ForestBanditRewardSourceEnum} source The source scene.
     */
    this.setRewardSource = function (source) {
        var history = global.kobold.history.get(id);
        history[rewardSource] = source;
        global.kobold.history.set(id, history);
    };

    /**
     * Fetches the reward source from history and returns it.
     * May return undefined if the source has not yet been set.
     * @returns {ForestBanditRewardSourceEnum} The source scene.
     */
    this.getRewardSource = function () {
        var history = global.kobold.history.get(id);
        return history[rewardSource];
    };

    /**
     * Returns true if the reward source has been set.
     */
    this.hasRewardSource = function () {
        return global.kobold.history.has(id, rewardSource);
    };

    /**
     * Reward continue.
     */
    this.rewardContinue = function () {
        var history = global.kobold.history.get(id);
        history[rewardId] = true;
        global.kobold.history.set(id, history);
    };

    /**
     * Returns true if the reward screen has continued.
     */
    this.hasRewardContinued = function () {
        return global.kobold.history.has(id, rewardId);
    };

    /**
     * The kobold's name at reward-time.
     * @returns {string} The kobold's name at reward-time.
     */
    this.rewardName = function () {
        var name = global.kobold.name;
        global.kobold.history.setIfNotSet(id, rewardNameId, name);
        return global.kobold.history.getKey(id, rewardNameId);
    };

    /**
     * Alters the kobold's inventory and stats and returns HTML describing the change.
     * @returns {string} HTML describing the change.
     */
    this.rewardScary = function () {
        global.kobold.history.doIfNotSet(id, rewardChangeId, true, function () {
            global.kobold.history.shared.dragonessPrincess.relation = DragonessPrincessEnum.SAVED;
            global.kobold.history.shared.dragonessPrincess.savedBy = DragonessPrincessSavedBy.SCARY_NOISE;
            global.kobold.status.mental.increase(50, "Saved the dragoness");
            global.kobold.inventory.bag.addMisc(new GoldenNeedle());
            global.kobold.inventory.equipment.weapon = new Rock();
        });
        return global.ui.change.buildNotification([
            new Change("Saved a princess", "Sanity", +50),
            new Change("Reward", global.kobold.inventory.createItemWithTooltip(new GoldenNeedle(), true), +1),
            new Change("Recovered rock", global.kobold.inventory.createItemWithTooltip(new Rock(), true), +1)
        ]);
    };
    
    /**
     * Alters the kobold's inventory and stats and returns HTML describing the change.
     * @returns {string} HTML describing the change.
     */
    function rewardPayRock() {
        var gold = rewardCoins();
        global.kobold.history.doIfNotSet(id, rewardChangeId, true, function () {
            global.kobold.history.shared.dragonessPrincess.relation = DragonessPrincessEnum.SAVED;
            global.kobold.history.shared.dragonessPrincess.savedBy = DragonessPrincessSavedBy.PAYING_GANG;
            global.kobold.history.shared.tenPawGang.relation = TenPawGangEnum.PAID;
            global.kobold.status.mental.increase(50, "Saved the dragoness");
            global.kobold.inventory.bag.addMisc(new GoldenNeedle());
            global.kobold.inventory.equipment.weapon = new Rock();
            global.kobold.inventory.coins = 0;
        });
        return global.ui.change.buildNotification([
            new Change("Saved a princess", "Sanity", +50),
            new Change("Reward", global.kobold.inventory.createItemWithTooltip(new GoldenNeedle(), true), +1),
            new Change("Recovered rock", global.kobold.inventory.createItemWithTooltip(new Rock(), true), +1),
            new Change("Paid", "Gold", -gold)
        ]);
    };
    
    /**
     * Alters the kobold's inventory and stats and returns HTML describing the change.
     * @returns {string} HTML describing the change.
     */
    function rewardPayNoRock() {
        var gold = rewardCoins();
        global.kobold.history.doIfNotSet(id, rewardChangeId, true, function () {
            global.kobold.history.shared.dragonessPrincess.relation = DragonessPrincessEnum.SAVED;
            global.kobold.history.shared.dragonessPrincess.savedBy = DragonessPrincessSavedBy.PAYING_GANG;
            global.kobold.history.shared.tenPawGang.relation = TenPawGangEnum.PAID;
            global.kobold.status.mental.increase(50, "Saved the dragoness");
            global.kobold.inventory.bag.addMisc(new GoldenNeedle());
            global.kobold.inventory.coins = 0;
        });
        return global.ui.change.buildNotification([
            new Change("Saved a princess", "Sanity", +50),
            new Change("Reward", global.kobold.inventory.createItemWithTooltip(new GoldenNeedle(), true), +1),
            new Change("Paid", "Gold", -gold)
        ]);
    };
    
    function rewardCoins() {
        var gold = global.kobold.inventory.coins;
        global.kobold.history.setIfNotSet(id, rewardGoldId, gold);
        return global.kobold.history.getKey(id, rewardGoldId);
    }
    
    /**
     * Alters the kobold's inventory and stats and returns HTML describing the change.
     * @returns {string} HTML describing the change.
     */
    this.rewardPay = function () {
        var didRock = (that.getArrivalChoice() === ForestBanditArrivalChoiceEnum.ROCK);
        if(didRock) {
            return rewardPayRock();
        } else {
            return rewardPayNoRock();
        }
    };
}
